package Perl::Critic::TooMuchCode;
our $VERSION="0.02";
1;
__END__

=head1 NAME

Perl::Critic::TooMuchCode - perlcritic add-ons that generally check for un-needed stuffs.

=head1 DESCRIPTION

Identifying what can be removed is often tedious and sometimes non-trivial to do. Bots can
probably do it better then humans.

=head1 AUTHOR

Kang-min Liu <gugod@gugod.org>

=head1 LICENSE

MIT

=cut
